/*****************************************************************************
 * Copyright (C) 2008  Eus (software_freedom at member dot fsf dot org)      *
 *                                                                           *
 * This program is free software: you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation, either version 3 of the License, or         *
 * (at your option) any later version.                                       *
 *                                                                           *
 * This program is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 * GNU General Public License for more details.                              *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.     *
 *****************************************************************************/

package phoneNumberMnemonics;

import java.util.NoSuchElementException;

/**
 * PhoneNumberMnemonics is an object that is able to generate all permutations
 * of letters from a given phone numbers. Phone numbers starting from 2 until 9
 * represents the following letters:
 * <ul>
 * <li>2 is abc</li>
 * <li>3 is def</li>
 * <li>4 is ghi</li>
 * <li>5 is jkl</li>
 * <li>6 is mno</li>
 * <li>7 is pqrs</li>
 * <li>8 is tuv</li>
 * <li>9 is wxyz</li>
 * </ul>
 * Therefore, a phone number of 2665 can be read as "cool", "aook", and so on,
 * which are the permutations of the letters represented by the numbers.
 *
 * @author Eus
 */
public class PhoneNumberMnemonics
{
	private WheelsPermutationMachine permutator = null;
	private boolean hasPermutatorRolledOver = false;
	private static Wheel [] numberToLetters = new Wheel [10];
	private String phoneNumber = null;

	/**
	 * Constructs an empty phone number mnemonics generator.
	 */
	public PhoneNumberMnemonics ()
	{
		numberToLetters [0] = new Wheel (new String [] {"0"});
		numberToLetters [1] = new Wheel (new String [] {"1"});
		numberToLetters [2] = new Wheel (new String [] {"a", "b", "c"});
		numberToLetters [3] = new Wheel (new String [] {"d", "e", "f"});
		numberToLetters [4] = new Wheel (new String [] {"g", "h", "i"});
		numberToLetters [5] = new Wheel (new String [] {"j", "k", "l"});
		numberToLetters [6] = new Wheel (new String [] {"m", "n", "o"});
		numberToLetters [7] = new Wheel (new String [] {"p", "q", "r"
									, "s"});
		numberToLetters [8] = new Wheel (new String [] {"t", "u", "v"});
		numberToLetters [9] = new Wheel (new String [] {"w", "x", "y"
									, "z"});
	}

	/**
	 * Constructs a phone number mnemonics generator that will generate a
	 * set of mnemonics based on the given phone number.
	 *
	 * @param phoneNumber the phone number from which the mnemonics shall
	 *                    be generated
	 *
	 * @throws IllegalArgumentException if phoneNumber is null or empty or
	 *                                  does not consist of only digit from
	 *                                  0 to 9
	 */
	public PhoneNumberMnemonics (String phoneNumber)
	{
		this ();

		setPhoneNumber (phoneNumber);
	}

	/**
	 * Sets new phone number from which the phone number mnemonics
	 * generator shall generate a set of mnemonics. This will reset the
	 * internal state of the generator.
	 *
	 * @param phoneNumber the phone number from which the mnemonics shall
	 *                    be generated
	 *
	 * @throws IllegalArgumentException if phoneNumber is null or empty or
	 *                                  does not consist of only digit from
	 *                                  0 to 9
	 */
	public void setPhoneNumber (String phoneNumber)
	{
		if (phoneNumber == null || phoneNumber.length () == 0)
		{
			throw new IllegalArgumentException (
				"phoneNumber is null or empty"
			);
		}

		Wheel [] tupleOfWheels = new Wheel [phoneNumber.length ()];
		int j = phoneNumber.length () - 1;

		for (int i = j; i >= 0; i--)
		{
			char nthChar = phoneNumber.charAt (j - i);

			if (!Character.isDigit (nthChar))
			{
				throw new IllegalArgumentException (
					"not all characters in phoneNumber"
								+ " is a number"
				);
			}

			tupleOfWheels [i] = numberToLetters [nthChar - '0'];
		}

		permutator = new WheelsPermutationMachine (tupleOfWheels);
		this.phoneNumber = phoneNumber;
		hasPermutatorRolledOver = false;
	}

	/**
	 * Returns the phone number based on which the mnemonics is generated.
	 */
	public String getPhoneNumber ()
	{
		return phoneNumber;
	}

	/**
	 * Returns the next mnemonics that can be generated from the given
	 * phone number. A call to hasNextMnemonics () should be made to ensure
	 * that there is more mnemonics to be returned.
	 *
	 * @throws IllegalStateException if a phone number has not been set
	 * @throws NoSuchElementException if no more mnemonic can be generated
	 *                                further
	 */
	public String nextMnemonics ()
	{
		if (permutator == null)
		{
			throw new IllegalStateException (
				"A phone number from which the mnemonics shall"
					      + " be generated has not been set"
			);
		}

		if (hasPermutatorRolledOver)
		{
			throw new NoSuchElementException (
				"No more mnemonic to be returned"
			);
		}

		String [] result = permutator.read ();
		hasPermutatorRolledOver = permutator.turn ();

		StringBuilder sb = new StringBuilder (result.length);
		for (int i = result.length - 1; i >= 0; i--)
		{
			sb.append (result [i]);
		}

		return sb.toString ();
	}

	/**
	 * Returns true if more mnemonics can still be returned.
	 * Otherwise, false.
	 */
	public boolean hasNextMnemonics ()
	{
		return !hasPermutatorRolledOver;
	}

	/**
	 * Generates all mnemonics from a given phone number in args [0].
	 */
	public static void main (String args [])
	{
		if (args.length != 1)
		{
			System.err.println (
				"Please supply a valid phone number"
			);

			System.exit (1);
		}

		PhoneNumberMnemonics gen = new PhoneNumberMnemonics (args [0]);

		while (gen.hasNextMnemonics ())
		{
			System.out.println (gen.nextMnemonics ());
		}

		System.exit (0);
	}
}
