/*****************************************************************************
 * Copyright (C) 2008  Eus (software_freedom at member dot fsf dot org)      *
 *                                                                           *
 * This program is free software: you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation, either version 3 of the License, or         *
 * (at your option) any later version.                                       *
 *                                                                           *
 * This program is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 * GNU General Public License for more details.                              *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.     *
 *****************************************************************************/

package phoneNumberMnemonics;

/**
 * WheelsPermutationMachine is an object with N number of wheels. Each wheel is
 * indexed starting from 0 until (N - 1). In each turn of the machine, the
 * wheel with index 0 is turned once. If the wheel undergoes a roll over, the
 * next wheel will be turned once. Otherwise, the next wheel will stay the
 * same. If the next wheel also undergoes a roll over, the next wheel after
 * that wheel will be turned once. Otherwise, the next wheel after that wheel
 * will stay the same. And, so on. Hence, the machine permutes the wheels in
 * each of its turn.
 *
 * @author Eus
 */
public class WheelsPermutationMachine
{
	private Wheel [] wheels;

	/**
	 * Constructs the machine with certain number of wheels. This machine
	 * clones each wheel in wheels so that subsequent modification to
	 * wheels will not affect the internal state of this machine.
	 *
	 * @param wheels the wheels that this machine will permute
	 *
	 * @throws IllegalArgumentException if wheels is null or empty
	 */
	public WheelsPermutationMachine (Wheel [] wheels)
	{
		if (wheels == null || wheels.length == 0)
		{
			throw new IllegalArgumentException (
				"wheels is null or empty"
			);
		}

		this.wheels = new Wheel [wheels.length];
		for (int i = 0; i < wheels.length; i++)
		{
			this.wheels [i] = (Wheel) wheels [i].clone ();
		}
		reset ();
	}

	/**
	 * Advances the wheel with index 0 once.
	 *
	 * @return true if the wheel with the largest index undergoes a roll
	 *         over. Otherwise, false.
	 */
	public boolean turn ()
	{
		int i = 0;
		boolean isRolledOver = false;

		for (; i < wheels.length; i++)
		{
			isRolledOver = wheels [i].turn ();

			if (!isRolledOver)
			{
				break;
			}
		}

		return (i == wheels.length);
	}

	/**
	 * Reads the markings of the wheels starting from the largest index to
	 * index 0.
	 *
	 * @return the markings of the wheels
	 */
	public String [] read ()
	{
		String [] result = new String [wheels.length];

		for (int i = wheels.length - 1; i >= 0; i--)
		{
			result [i] = wheels [i].read ();
		}

		return result;
	}

	/**
	 * Resets this machine.
	 */
	public void reset ()
	{
		for (int i = 0; i < wheels.length; i++)
		{
			wheels [i].reset ();
		}
	}
}
